@@ -1,6 +1,6 @@
 # pass_widget.py
 #
-# Copyright 2022 Pablo Sánchez Rodríguez
+# Copyright 2022-2023 Pablo Sánchez Rodríguez
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,25 +18,13 @@
 from gi.repository import Adw, Gdk, Graphene, Gsk, Gtk, Pango
 
 from .barcode_widget import BarcodeWidget
+from .digital_pass import Color
 
 
 PASS_WIDTH = 320
 PASS_HEIGHT = 420
 PASS_MARGIN = 12
-
-
-class PassColor:
-    white = Gdk.RGBA()
-    white.red = 1
-    white.blue = 1
-    white.green = 1
-    white.alpha = 1
-
-    black = Gdk.RGBA()
-    black.red = 0
-    black.blue = 0
-    black.green = 0
-    black.alpha = 1
+BACKGROUND_BLUR_RADIUS = 30
 
 
 class PassFont:
@@ -118,14 +106,14 @@ class PassPlotter:
         self._pango_context = pass_widget.get_pango_context()
 
         # Barcode
-        self._barcode = BarcodeWidget()
-        barcodes = a_pass.barcodes()
-        if barcodes:
+        self._barcode = None
+        barcode = a_pass.barcodes()[0]
+        if barcode:
+            self._barcode = BarcodeWidget()
             self._barcode.props.width_request = PASS_WIDTH
             self._barcode.props.height_request = 110
             self._pass_widget.put(self._barcode, 0, PASS_HEIGHT - PASS_MARGIN - self._barcode.props.height_request)
 
-            barcode = barcodes[0]
             self._barcode.encode(barcode.format(),
                                  barcode.message(),
                                  barcode.message_encoding())
@@ -240,23 +228,18 @@ class EsPassPlotter(PassPlotter):
         espass = a_pass.adaptee()
 
         # Accent color
-        self._accent_color = Gdk.RGBA()
         accent_color = espass.accent_color()
-        if accent_color:
-            self._accent_color.red = accent_color.red() / 255
-            self._accent_color.blue = accent_color.blue() / 255
-            self._accent_color.green = accent_color.green() / 255
-            self._accent_color.alpha = 1.0
+        self._accent_color = accent_color.as_gdk_rgba() \
+            if accent_color else Gdk.RGBA()
 
         # Background color
-        self._bg_color = PassColor.white
+        self._bg_color = Color.named('white').as_gdk_rgba()
 
         # Foreground color
-        self._fg_color = PassColor.black
+        self._fg_color = Color.named('black').as_gdk_rgba()
 
         # Label color
         self._label_color = self._fg_color.copy()
-        self._label_color.alpha = 0.5
 
         # Logo
         self._logo_texture  = None
@@ -270,7 +253,7 @@ class EsPassPlotter(PassPlotter):
     def _plot_background(self):
         rectangle = Graphene.Rect()
         rectangle.init(0, 0, PASS_WIDTH, PASS_HEIGHT)
-        self._snapshot.append_color(PassColor.white, rectangle)
+        self._snapshot.append_color(self._bg_color, rectangle)
 
     def _plot_fields(self):
         self._plot_fields_layouts(self._fields)
@@ -305,11 +288,12 @@ class EsPassPlotter(PassPlotter):
         self._plot_fields()
         self._snapshot.restore()
 
-        #self._plot_barcode()
-
 
 class PkPassPlotter(PassPlotter):
 
+    PRIMARY_FIELD_LABEL_FONT = PassFont.label
+    PRIMARY_FIELD_VALUE_FONT = PassFont.biggest_value
+
     def __init__(self, a_pass, pass_widget):
         super().__init__(a_pass, pass_widget)
 
@@ -318,36 +302,35 @@ class PkPassPlotter(PassPlotter):
 
         # Background color
         bg_color = pkpass.background_color()
-        if bg_color:
-            self._bg_color = Gdk.RGBA()
-            self._bg_color.red = bg_color.red() / 255
-            self._bg_color.blue = bg_color.blue() / 255
-            self._bg_color.green = bg_color.green() / 255
-            self._bg_color.alpha = 1.0
-        else:
-            self._bg_color = PassColor.white
+        self._bg_color = bg_color.as_gdk_rgba() \
+            if bg_color else Color.named('white').as_gdk_rgba()
 
         # Foreground color
         fg_color = pkpass.foreground_color()
-        if fg_color:
-            self._fg_color = Gdk.RGBA()
-            self._fg_color.red = fg_color.red() / 255
-            self._fg_color.blue = fg_color.blue() / 255
-            self._fg_color.green = fg_color.green() / 255
-            self._fg_color.alpha = 1.0
-        else:
-            self._fg_color = PassColor.black
+        self._fg_color = fg_color.as_gdk_rgba() \
+            if fg_color else Color.named('black').as_gdk_rgba()
 
         # Label color
-        self._label_color = self._fg_color.copy()
-        self._label_color.alpha = 0.5
+        label_color = pkpass.label_color()
+        if label_color:
+            self._label_color = label_color.as_gdk_rgba()
+        else:
+            self._label_color = self._fg_color.copy()
 
-        # Logo
+        # Images
+        self._background_texture = None
         self._logo_texture = None
+        self._strip_texture = None
+
+        if pkpass.background():
+            self._background_texture = pkpass.background().as_texture()
 
         if pkpass.logo():
             self._logo_texture = pkpass.logo().as_texture()
 
+        if pkpass.strip():
+            self._strip_texture = pkpass.strip().as_texture()
+
         # Fields
         self._header_fields = pkpass.header_fields()
         self._primary_fields = pkpass.primary_fields()
@@ -378,8 +361,6 @@ class PkPassPlotter(PassPlotter):
         self._plot_secondary_and_axiliary_fields()
         self._snapshot.restore()
 
-        self._plot_barcode()
-
     def _plot_header(self):
         header_height = 32
 
@@ -429,14 +410,65 @@ class PkPassPlotter(PassPlotter):
     def _plot_footer(self):
         raise NotImplementedError
 
-    def _plot_barcode(self):
-        rectangle = Graphene.Rect()
-        rectangle.init(0,
-                       PASS_HEIGHT - PASS_MARGIN - self._barcode.props.height_request,
-                       PASS_WIDTH,
-                       self._barcode.props.height_request)
 
-        self._snapshot.append_color(PassColor.white, rectangle)
+class PkPassWithStripPlotter(PkPassPlotter):
+    """
+    PkPassWithStripPlotter is a PkPassPlotter for PKPasses that may contain a
+    strip image.
+    """
+
+    STRIP_IMAGE_MAX_HEIGHT = 123
+
+    def __init__(self, pkpass, pkpass_widget):
+        super().__init__(pkpass, pkpass_widget)
+
+    def _plot_primary_fields(self):
+
+        # Draw the strip
+
+        strip_height = 0
+
+        if self._strip_texture and not self._background_texture:
+            strip_scale = PASS_WIDTH / self._strip_texture.get_width()
+            strip_height = self._strip_texture.get_height() * strip_scale
+
+            rectangle = Graphene.Rect()
+            rectangle.init(0, -PASS_MARGIN, PASS_WIDTH, strip_height)
+
+            strip_height = min(self.STRIP_IMAGE_MAX_HEIGHT, strip_height)
+            strip_area = Graphene.Rect()
+            strip_area.init(0, -PASS_MARGIN, PASS_WIDTH, strip_height)
+
+            self._snapshot.push_clip(strip_area)
+            self._snapshot.append_texture(self._strip_texture, rectangle)
+            self._snapshot.pop()
+
+        # Draw the primary fields
+
+        point = Graphene.Point()
+        field_layout_height = 0
+
+        if self._primary_fields:
+            field_layout = FieldLayout(self._pango_context,
+                                       self._primary_fields[0],
+                                       value_font = self.PRIMARY_FIELD_VALUE_FONT)
+
+            field_layout_height = field_layout.get_height()
+            self._snapshot.save()
+
+            point.x = PASS_MARGIN
+            point.y = 0
+            self._snapshot.translate(point)
+
+            field_layout.append(self._snapshot, self._label_color, self._fg_color)
+
+            self._snapshot.restore()
+
+        # Perform a translation so that the next drawing starts below this one
+        point.x = 0
+        point.y = strip_height if strip_height > field_layout_height \
+                               else field_layout_height + 2 * PASS_MARGIN
+        self._snapshot.translate(point)
 
 
 class BoardingPassPlotter(PkPassPlotter):
@@ -481,69 +513,41 @@ class BoardingPassPlotter(PkPassPlotter):
         self._plot_fields_layouts(self._secondary_fields)
 
 
-class CouponPlotter(PkPassPlotter):
+class CouponPlotter(PkPassWithStripPlotter):
+
+    STRIP_IMAGE_MAX_HEIGHT = 144
 
     def __init__(self, pkpass, pkpass_widget):
         super().__init__(pkpass, pkpass_widget)
 
-    def _plot_primary_fields(self):
-        if not self._primary_fields:
-            return
-
-        field_layout = FieldLayout(self._pango_context,
-                                   self._primary_fields[0],
-                                   value_font = PassFont.biggest_value,
-                                   alignment = Pango.Alignment.LEFT)
-
-        self._snapshot.save()
-
-        point = Graphene.Point()
-        point.x = PASS_MARGIN
-        point.y = 0
-        self._snapshot.translate(point)
-
-        field_layout.append(self._snapshot, self._label_color, self._fg_color)
-
-        self._snapshot.restore()
-
-        # Perform a translation so that the next drawing starts below this one
-        point.x = 0
-        point.y = field_layout.get_height() + 2 * PASS_MARGIN
-        self._snapshot.translate(point)
-
     def _plot_secondary_and_axiliary_fields(self):
         self._plot_fields_layouts(self._secondary_fields + \
                                   self._auxiliary_fields)
 
 
-class EventTicketPlotter(PkPassPlotter):
+class EventTicketPlotter(PkPassWithStripPlotter):
+
+    PRIMARY_FIELD_VALUE_FONT = PassFont.big_value
+    STRIP_IMAGE_MAX_HEIGHT = 98
 
     def __init__(self, pkpass, pkpass_widget):
         super().__init__(pkpass, pkpass_widget)
 
-    def _plot_primary_fields(self):
-        if not self._primary_fields:
-            return
+    def _plot_background(self):
 
-        field_layout = FieldLayout(self._pango_context,
-                                   self._primary_fields[0],
-                                   value_font = PassFont.big_value)
+        if not self._strip_texture and self._background_texture:
+            rectangle = Graphene.Rect()
+            rectangle.init(-BACKGROUND_BLUR_RADIUS,
+                           -BACKGROUND_BLUR_RADIUS,
+                           PASS_WIDTH + 2 * BACKGROUND_BLUR_RADIUS,
+                           PASS_HEIGHT + 2 * BACKGROUND_BLUR_RADIUS)
 
-        self._snapshot.save()
+            self._snapshot.push_blur(BACKGROUND_BLUR_RADIUS)
+            self._snapshot.append_texture(self._background_texture, rectangle)
+            self._snapshot.pop()
 
-        point = Graphene.Point()
-        point.x = PASS_MARGIN
-        point.y = 0
-        self._snapshot.translate(point)
-
-        field_layout.append(self._snapshot, self._label_color, self._fg_color)
-
-        self._snapshot.restore()
-
-        # Perform a translation so that the next drawing starts below this one
-        point.x = 0
-        point.y = field_layout.get_height() + 2 * PASS_MARGIN
-        self._snapshot.translate(point)
+        else:
+            super()._plot_background()
 
     def _plot_secondary_and_axiliary_fields(self):
         self._plot_fields_layouts(self._secondary_fields + \
@@ -602,6 +606,8 @@ class PassWidget(Gtk.Fixed):
         self.props.halign = Gtk.Align.CENTER
         self.props.valign = Gtk.Align.CENTER
 
+        self.props.focusable = True
+
         self.add_css_class('card')
 
     def do_snapshot(self, snapshot):
